@@ -1,5 +1,5 @@
 # Part of the PsychoPy library
-# Copyright (C) 2010 Jonathan Peirce
+# Copyright (C) 2011 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 from _base import *
@@ -11,7 +11,7 @@ iconFile = path.join(thisFolder,'mouse.png')
 
 class MouseComponent(BaseComponent):
     """An event class for checking the mouse location and buttons at given timepoints"""
-    def __init__(self, exp, parentName, name='mouse', startTime=0.0, duration=1.0, save='final', 
+    def __init__(self, exp, parentName, name='mouse', startTime=0.0, duration='', save='final', 
             forceEndTrialOnPress=True, timeRelativeTo='routine'):
         self.type='Mouse'
         self.url="http://www.psychopy.org/builder/components/mouse.html"
@@ -57,7 +57,6 @@ class MouseComponent(BaseComponent):
         """Write the code that will be called every frame
         """
         forceEnd = self.params['forceEndTrialOnPress'].val
-        continueName = self.exp.flow._currentRoutine._continueName
         routineClockName = self.exp.flow._currentRoutine._clockName
         
         #only write code for cases where we are storing data as we go (each frame or each click)
@@ -89,7 +88,7 @@ class MouseComponent(BaseComponent):
         #does the response end the trial?
         if forceEnd==True:
             buff.writeIndented("#abort routine on response\n" %self.params)
-            buff.writeIndented("%s=False\n" %continueName)
+            buff.writeIndented("continueRoutine=False\n")
             
         #dedent
         if self.params['saveMouseState'] == 'on click' or forceEnd:
