@@ -1,5 +1,5 @@
 # Part of the PsychoPy library
-# Copyright (C) 2010 Jonathan Peirce
+# Copyright (C) 2011 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 import urllib2, socket, re, glob
@@ -79,8 +79,13 @@ class Updater:
         #parse update file as a dictionary
         latest={}
         for line in page.readlines():
+            #in some odd circumstances (wifi hotspots) you might successfully fetch a 
+            #page that is not the correct URL but a redirect
+            if line.find(':')==-1:
+                return -1
+            #this will succeed if every line has a key
             key, keyInfo = line.split(':')
-            latest[key]=keyInfo.replace('\n', '')
+            latest[key]=keyInfo.replace('\n', '').replace('\r', '')
         return latest
     def suggestUpdate(self, confirmationDlg=False):
         """Query user about whether to update (if it's possible to do the update)
@@ -186,10 +191,10 @@ class InstallUpdateDialog(wx.Dialog):
         """
         self.app = app
         #get latest version info if poss
-        if app.updater==None:
+        if app.updater in [False,None]:
             #user has turned off check for updates in prefs so check now
             app.updater = updater = Updater(app=self.app)
-            self.latest=updater.getLatestUpdate(warnMsg=False)#don't need a warning - we'll provide one ourselves
+            self.latest=updater.getLatestInfo(warnMsg=False)#don't need a warning - we'll provide one ourselves
         else:
             self.latest=app.updater.latest
         self.runningVersion=app.updater.runningVersion
@@ -243,14 +248,14 @@ class InstallUpdateDialog(wx.Dialog):
                 "You can revert to a previous version by selecting a specific .zip source installation file" 
         else:
             msg = "PsychoPy v%s is available\nYou are running v%s" %(self.latest['version'], self.runningVersion)
-            if self.latest['lastCompatible']<self.runningVersion:
+            if self.latest['lastUpdatable']<=self.runningVersion:
                 msg+="\nYou can update to the latest version automatically"
             else:
                 msg+="\nYou cannot update to the latest version automatically.\nPlease fetch the latest Standalone package from www.psychopy.org"  
         self.statusMessage.SetLabel(msg)
         if self.latest==-1 \
             or self.latest['version']==self.runningVersion \
-            or self.latest['lastCompatible']>self.runningVersion:#can't auto-update
+            or self.latest['lastUpdatable']>self.runningVersion:#can't auto-update
                 self.currentSelection=self.useZipBtn
                 self.useZipBtn.SetValue(True)
                 self.useLatestBtn.Disable()
@@ -275,6 +280,7 @@ class InstallUpdateDialog(wx.Dialog):
             self.progressBar.Disable()
             self.installBtn.Enable()#if this has been disabled by the fact that we couldn't connect
     def onCancel(self, event):
+        self.app.updater=None
         self.Destroy()
     def onFileBrowse(self, event):
         self.filename = event.GetString()
@@ -327,7 +333,7 @@ class InstallUpdateDialog(wx.Dialog):
                 if searchName!=None:
                     v=searchName.group(0)[:-1]
                 else:log.warning("Couldn't deduce version from zip file: %s" %zFilename)
-            f=open(zfile)
+            f=open(zfile, 'rb')
             zfile=zipfile.ZipFile(f)
         else:#assume here that zfile is a ZipFile
             pass#todo: error checking - is it a zipfile?
@@ -344,7 +350,11 @@ class InstallUpdateDialog(wx.Dialog):
                 os.rename(currPath, "%s-%s" %(currPath, psychopy.__version__))
                 undoString += 'os.rename("%s-%s" %(currPath, psychopy.__version__),currPath)\n'
             except:
-                return "Could not move existing PsychoPy installation (permissions error?)"
+                if sys.platform=='win32' and int(sys.getwindowsversion()[1])>5:
+                    msg = "Right-click the app and 'Run as admin' to upgrade)"
+                else:
+                    msg = "Could not move existing PsychoPy installation (permissions error?)"
+                return msg
         else:#setuptools-style installation
             #generate new target path
             unzipTarget=currPath
@@ -364,7 +374,11 @@ class InstallUpdateDialog(wx.Dialog):
             undoString += 'os.remove(%s)\n' %unzipTarget
         except: #revert path rename and inform user
             exec(undoString)#undo previous changes
-            return ("Could not create new path (permissions error?):\n%s" %unzipTarget)
+            if sys.platform=='win32' and int(sys.getwindowsversion()[1])>5:
+                msg = "Right-click the app and 'Run as admin'):\n%s" %unzipTarget
+            else:
+                msg = "Failed to create directory for new version (permissions error?):\n%s" %unzipTarget
+            return msg
 
         #do the actual extraction
         for name in zfile.namelist():#for each file within the zip
@@ -458,7 +472,7 @@ def sendUsageStats(proxy=None):
     if sys.platform=='darwin':
         OSXver, junk, architecture = platform.mac_ver()
         systemInfo = "OSX_%s_%s" %(OSXver, architecture)
-    elif sys.platform=='linux':
+    elif sys.platform.startswith('linux'):
         systemInfo = '%s_%s_%s' % (
             'Linux',
             ':'.join([x for x in platform.dist() if x != '']),
@@ -466,7 +480,7 @@ def sendUsageStats(proxy=None):
     elif sys.platform=='win32':
         ver=sys.getwindowsversion()
         if len(ver[4])>0:
-            systemInfo="win32_v%i.%i.%i (%s)" %(ver[0],ver[1],ver[2],ver[4])
+            systemInfo=("win32_v%i.%i.%i_%s" %(ver[0],ver[1],ver[2],ver[4])).replace(' ','')
         else:
             systemInfo="win32_v%i.%i.%i" %(ver[0],ver[1],ver[2])
     else:
@@ -479,3 +493,37 @@ def sendUsageStats(proxy=None):
     except:
         log.warning("Couldn't connect to psychopy.org\n"+\
             "Check internet settings (and proxy setting in PsychoPy Preferences.")
+
+def getAutoProxyURL():
+    """Return a list of possible auto proxy .pac files being used, 
+    based on the system registry (win32) or system preferences (OSX).
+    """
+    pacFiles=[]
+    if sys.platform=='win32':
+        try:
+            import _winreg as winreg#used from python 2.0-2.6
+        except:
+            import winreg#used from python 2.7 onwards
+        net = winreg.OpenKey(
+            winreg.HKEY_CURRENT_USER,
+            "Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"
+            )
+        nSubs, nVals, lastMod = winreg.QueryInfoKey(net)
+        subkeys={}
+        for i in range(nVals):
+            thisName, thisVal, thisType=winreg.EnumValue(net,i)
+            subkeys[thisName]=thisVal
+        if 'AutoConfigURL' in subkeys.keys() and len(subkeys['AutoConfigURL'])>0:
+            pacFiles.append(subkeys['AutoConfigURL'])
+    elif sys.platform=='darwin':
+        import plistlib
+        sysPrefs = plistlib.readPlist('/Library/Preferences/SystemConfiguration/preferences.plist')
+        networks=sysPrefs['NetworkServices']
+        pacFiles = []
+        #loop through each possible network (e.g. Ethernet, Airport...)
+        for network in networks.items():
+            netKey, network=network#the first part is a long identifier
+            if 'ProxyAutoConfigURLString' in network['Proxies'].keys():
+                pacFiles.append(network['Proxies']['ProxyAutoConfigURLString'])
+    return pacFiles
+        
\ No newline at end of file
