@@ -1,13 +1,13 @@
 #!/usr/bin/env python
 
 # Part of the PsychoPy library
-# Copyright (C) 2010 Jonathan Peirce
+# Copyright (C) 2011 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 import sys, psychopy
-import StringIO
+import StringIO, copy
 if sys.argv[-1] in ['-v', '--version']:
-    print 'PsychoPy2, version %s (c)Jonathan Peirce, 2010, GNU GPL license' %psychopy.__version__
+    print 'PsychoPy2, version %s (c)Jonathan Peirce, 2011, GNU GPL license' %psychopy.__version__
     sys.exit()
 if sys.argv[-1] in ['-h', '--help']:
     print """Starts the PsychoPy2 application.
@@ -39,7 +39,7 @@ if not hasattr(sys, 'frozen'):
 import wx
 #NB keep imports to a minimum here because splash screen has not yet shown
 #e.g. coder and builder are imported during app.__init__ because they take a while
-from psychopy import preferences#needed by splash screen for the path to resources/psychopySplash.png
+from psychopy import preferences, log#needed by splash screen for the path to resources/psychopySplash.png
 import sys, os, threading, time, platform
 
 # get UID early; psychopy should never need anything except plain-vanilla user
@@ -120,6 +120,8 @@ class PsychoPyApp(wx.App):
         from psychopy.app import coder, builder, wxIDs, connections, urls
         #set default paths and prefs
         self.prefs = preferences.Preferences() #from preferences.py
+        if self.prefs.app['debugMode']:
+            log.console.setLevel(log.DEBUG)
         self.keys = self.prefs.keys
         self.prefs.pageCurrent = 0  # track last-viewed page of prefs, to return there
         self.IDs=wxIDs
@@ -140,8 +142,14 @@ class PsychoPyApp(wx.App):
             else:
                 self.prefs.app['defaultView'] = 'both'
                 mainFrame = 'both'
-        #then override the main frame by command options and passed files
-        scripts=[]; exps=[]
+        #fetch prev files if that's the preference
+        if self.prefs.coder['reloadPrevFiles']:
+            scripts=self.prefs.appData['coder']['prevFiles']
+        else: scripts=[]
+        if self.prefs.builder['reloadPrevExp'] and ('prevFiles' in self.prefs.appData['builder'].keys()):
+            exps=self.prefs.appData['builder']['prevFiles']
+        else: exps=[]
+        #then override the prev files by command options and passed files
         if len(sys.argv)>1:
             if sys.argv[1]==__name__:
                 args = sys.argv[2:] # program was excecuted as "python.exe PsychoPyIDE.py %1'
@@ -168,7 +176,10 @@ class PsychoPyApp(wx.App):
         if not (50<self.dpi<120): self.dpi=80#dpi was unreasonable, make one up
 
         #create both frame for coder/builder as necess
-        self.coder = self.builder = None
+        self.coder = None
+        self.builderFrames = []
+        self.copiedRoutine=None
+        self.allFrames=[]#these are ordered and the order is updated with self.onNewTopWindow
         if mainFrame in ['both', 'coder']: self.showCoder(fileList=scripts)
         if mainFrame in ['both', 'builder']: self.showBuilder(fileList=exps)
 
@@ -181,26 +192,21 @@ class PsychoPyApp(wx.App):
             self.updater=connections.Updater(app=self, proxy=self.prefs.connections['proxy'])
             self.updater.suggestUpdate(confirmationDlg=False)#check for updates (silently)
         else: self.updater=False
-        """This is in wx demo. Probably useful one day.
-        #---------------------------------------------
-        def ShowTip(self):
-            config = GetConfig()
-            showTipText = config.Read("tips")
-            if showTipText:
-                showTip, index = eval(showTipText)
-            else:
-                showTip, index = (1, 0)
-
-            if showTip:
-                tp = wx.CreateFileTipProvider(opj("data/tips.txt"), index)
-                ##tp = MyTP(0)
-                showTip = wx.ShowTip(self, tp)
-                index = tp.GetCurrentTip()
-                config.Write("tips", str( (showTip, index) ))
-                config.Flush()"""
+        
+        if self.prefs.app['showStartupTips']:
+            tipIndex = self.prefs.appData['tipIndex']            
+            tp = wx.CreateFileTipProvider(os.path.join(self.prefs.paths['resources'],"tips.txt"), tipIndex)
+            showTip = wx.ShowTip(None, tp)
+            self.prefs.appData['tipIndex'] = tp.GetCurrentTip()
+            self.prefs.saveAppData()
+            self.prefs.app['showStartupTips'] = showTip
+            self.prefs.saveUserPrefs()
 
         return True
-
+    def getPrimaryDisplaySize(self):
+        """Get the size of the primary display (whose coords start (0,0))
+        """
+        return list(wx.Display(0).GetGeometry())[2:]
     def showCoder(self, event=None, fileList=None):
         from psychopy.app import coder#have to reimport because it is ony local to __init__ so far
         if self.coder==None:
@@ -211,56 +217,123 @@ class PsychoPyApp(wx.App):
         self.SetTopWindow(self.coder)
         self.coder.Raise()
         self.coder.setOutputWindow()#takes control of sys.stdout
-    def showBuilder(self, event=None, fileList=None):
+        self.allFrames.append(self.coder)
+    def newBuilderFrame(self, event=None, fileName=None):
         from psychopy.app import builder#have to reimport because it is ony local to __init__ so far
-        if self.builder==None:
-            self.builder=builder.BuilderFrame(None, -1,
-                                  title="PsychoPy2 Experiment Builder",
-                                  files = fileList, app=self)
-        self.builder.Show(True)
-        self.builder.Raise()
-        self.SetTopWindow(self.builder)
+        thisFrame = builder.BuilderFrame(None, -1,
+                                  title="PsychoPy2 Experiment Builder (v%s)" %self.version,
+                                  fileName=fileName, app=self)
+        thisFrame.Show(True)
+        thisFrame.Raise()
+        self.SetTopWindow(thisFrame)
+        self.builderFrames.append(thisFrame)
+        self.allFrames.append(thisFrame)
+    def showBuilder(self, event=None, fileList=[]):
+        from psychopy.app import builder#have to reimport because it is ony local to __init__ so far
+        for fileName in fileList:
+            if os.path.isfile(fileName):
+                self.newBuilderFrame(fileName=fileName)
+        #create an empty Builder view if needed
+        if len(self.builderFrames)==0:
+            self.newBuilderFrame()
+        #loop through all frames, from the back bringing each forward
+        for thisFrame in self.allFrames:
+            if thisFrame.frameType!='builder':continue
+            thisFrame.Show(True)
+            thisFrame.Raise()
+            self.SetTopWindow(thisFrame)
+    #def showShell(self, event=None):
+    #    from psychopy.app import ipythonShell#have to reimport because it is ony local to __init__ so far
+    #    if self.shell==None:
+    #        self.shell = ipythonShell.ShellFrame(None, -1, 
+    #            title="IPython in PsychoPy (v%s)" %self.version, app=self)
+    #        self.shell.Show()
+    #        self.shell.SendSizeEvent()
+    #    self.shell.Raise()
+    #    self.SetTopWindow(self.shell)                           
+    #    self.shell.SetFocus()
     def openUpdater(self, event=None):
         from psychopy.app import connections
         dlg = connections.InstallUpdateDialog(parent=None, ID=-1, app=self)
         
+    def colorPicker(self, event=None):
+        """Opens system color-picker, sets clip-board and parent.new_rgb = string [r,g,b].
+        
+        Note: units are psychopy -1..+1 rgb units to three decimal places, preserving 24-bit color
+        """
+        class ColorPicker(wx.Panel):
+            def __init__(self, parent):
+                wx.Panel.__init__(self, parent, wx.ID_ANY)
+                rgb = 'None'
+                dlg = wx.ColourDialog(self)
+                dlg.GetColourData().SetChooseFull(True)
+                if dlg.ShowModal() == wx.ID_OK:
+                    data = dlg.GetColourData()
+                    rgb = data.GetColour().Get()
+                    rgb = map(lambda x: "%.3f" % ((x-127.5)/127.5),list(rgb))
+                    rgb = '['+','.join(rgb)+']'
+                    if wx.TheClipboard.Open():
+                        #http://wiki.wxpython.org/AnotherTutorial#wx.TheClipboard
+                        wx.TheClipboard.Clear()
+                        wx.TheClipboard.SetData(wx.TextDataObject(str(rgb)))
+                        wx.TheClipboard.Close()
+                dlg.Destroy()
+                parent.new_rgb = rgb        
+        frame = wx.Frame(None, wx.ID_ANY, "Color picker", size=(0,0)) # not shown
+        ColorPicker(frame)
+        new_rgb = frame.new_rgb # string; also on system clipboard, try wx.TheClipboard
+        frame.Destroy()
+        return new_rgb
     def openMonitorCenter(self,event):
         from psychopy.monitors import MonitorCenter
         frame = MonitorCenter.MainFrame(None,'PsychoPy2 Monitor Center')
         frame.Show(True)
     def MacOpenFile(self,fileName):
+        log.debug('PsychoPyApp: Received Mac file dropped event')
         if fileName.endswith('.py'):
             self.coder.setCurrentDoc(fileName)
         elif fileName.endswith('.psyexp'):
-            self.builder.fileOpen(filename=fileName)
+            self.newBuilderFrame(fileName=fileName)
     def quit(self, event=None):
+        log.debug('PsychoPyApp: Quitting...')
         self.quitting=True
         #see whether any files need saving
-        for frame in [self.coder, self.builder]:
-            if frame==None: continue
-            ok=frame.checkSave()
-            if not ok: return#user cancelled quit
+        for frame in self.allFrames:
+            try:#will fail if the frame has been shut somehow elsewhere
+                ok=frame.checkSave()
+            except: 
+                ok=False
+            if not ok: 
+                log.debug('PsychoPyApp: User cancelled shutdown')
+                return#user cancelled quit
+            
         #save info about current frames for next run
-        if self.coder and not self.builder:
+        if self.coder and len(self.builderFrames)==0:
             self.prefs.appData['lastFrame']='coder'
-        elif self.builder and not self.coder:
+        elif self.coder==None:
             self.prefs.appData['lastFrame']='builder'
         else:
             self.prefs.appData['lastFrame']='both'
-        #hide the frames then close
-        for frame in [self.coder, self.builder]:
+        
+        #update app data while closing each frame
+        self.prefs.appData['builder']['prevFiles']=[]#start with an empty list to be appended by each frame
+        self.prefs.appData['coder']['prevFiles']=[]
+        for frame in copy.copy(self.allFrames):
             if frame==None: continue
             frame.closeFrame(checkSave=False)#should update (but not save) prefs.appData
             self.prefs.saveAppData()#must do this before destroying the frame?
-            frame.Destroy()#because closeFrame actually just Hides the frame
         if sys.platform=='darwin':
             self.menuFrame.Destroy()
+            
         sys.exit()#really force a quit
+        
     def showPrefs(self, event):
+        log.debug('PsychoPyApp: Showing prefs dlg')
         prefsDlg = preferences.PreferencesDlg(app=self)
         prefsDlg.Show()
 
     def showAbout(self, event):
+        log.debug('PsychoPyApp: Showing about dlg')
 
         licFile = open(os.path.join(self.prefs.paths['psychopy'],'LICENSE.txt'))
         license = licFile.read()
@@ -275,7 +348,7 @@ let me/us know at psychopy-users@googlegroups.com"""
         info.SetVersion('v'+psychopy.__version__)
         info.SetDescription(msg)
 
-        info.SetCopyright('(C) 2002-2010 Jonathan Peirce')
+        info.SetCopyright('(C) 2002-2011 Jonathan Peirce')
         info.SetWebSite('http://www.psychopy.org')
         info.SetLicence(license)
         info.AddDeveloper('Jonathan Peirce')
