@@ -21,10 +21,13 @@ addRoutineToFlow=wx.NewId()
 addLoopToFlow=wx.NewId()
 remRoutineFromFlow=wx.NewId()
 remLoopFromFlow=wx.NewId()
+copyRoutine=wx.NewId()
+pasteRoutine=wx.NewId()
 
 #view menu
 openCoderView = wx.NewId()
 openBuilderView = wx.NewId()
+openShell = wx.NewId()
 toggleOutput=wx.NewId()
 toggleSourceAsst=wx.NewId()
 toggleIndentGuides=wx.NewId()
@@ -40,6 +43,7 @@ runFile=wx.NewId()
 stopFile=wx.NewId()
 monitorCenter=wx.NewId()
 openUpdater=wx.NewId()
+unitTests=wx.NewId()
 
 #help menu
 #these should be assigned to the relevant buttons/menu items in the app
@@ -50,6 +54,8 @@ coderTutorial=wx.NewId()
 builderHelp=wx.NewId()
 psychopyHome=wx.NewId()
 psychopyReference=wx.NewId()
+builderDemosUnpack=wx.NewId()
+builderDemos=wx.NewId()
 #help pages (from help buttons)
 docsPrefsDlg=wx.NewId()
 
@@ -66,4 +72,4 @@ tbCompile=120
 tbPreferences=130#for the app
 tbExpSettings=140#for the experiment
 tbMonitorCenter=150
-
+tbColorPicker=160
