@@ -3,7 +3,7 @@ Various useful functions for creating filters and textures (e.g. for PatchStim)
 	
 """
 # Part of the PsychoPy library
-# Copyright (C) 2010 Jonathan Peirce
+# Copyright (C) 2011 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 import numpy
@@ -31,16 +31,18 @@ def makeGrating(res,
             gratType: 'sin', 'sqr', 'ramp' or 'sinXsin' (default="sin")
                 the type of grating to be 'drawn'
             contr: float (default=1.0)
-            
+                contrast of the grating
+                
         :Returns:
             a square numpy array of size resXres
+            
     """
-    
-    ori *= (numpy.pi/180)
+    tiny=0.0000000000001#to prevent the sinusoid ever being exactly at zero (for sqr wave)
+    ori *= (-numpy.pi/180)
     phase *= (numpy.pi/180)
-    cycle1D = numpy.arange(0.0,cycles*2.0*numpy.pi,cycles*2.0*numpy.pi/res),
     xrange, yrange = numpy.mgrid[0.0 : cycles*2.0*numpy.pi : cycles*2.0*numpy.pi/res,
-                                                             0.0 : cycles*2.0*numpy.pi : cycles*2.0*numpy.pi/res]
+                                    0.0 : cycles*2.0*numpy.pi : cycles*2.0*numpy.pi/res]
+
     if gratType is "none":
             res=2
             intensity = numpy.ones((res,res),Float)
@@ -49,7 +51,7 @@ def makeGrating(res,
     elif gratType is "ramp":
             intensity= contr*( xrange*numpy.cos(ori)+yrange*numpy.sin(ori) )/(2*numpy.pi)
     elif gratType is "sqr":#square wave (symmetric duty cycle)
-            intensity = numpy.where(yrange>=numpy.pi, 1, -1)
+            intensity = numpy.where(numpy.sin( xrange*numpy.sin(ori)+yrange*numpy.cos(ori) + phase + tiny)>=0, 1, -1)
     elif gratType is "sinXsin":
             intensity = numpy.sin(xrange)*numpy.sin(yrange)
     else:#might be a filename of an image
@@ -74,10 +76,11 @@ def maskMatrix(matrix, shape='circle', radius=1.0, center=(0.0,0.0)):
             center:  2x1 tuple or list (default=[0.0,0.0])
                 the centre of the mask in the matrix ([1,1] is top-right corner, [-1,-1] is bottom-left)
     """
-    alphaMask = makeMask(matrix.shape[0],shape,radius, center=(0.0,0.0))
+    #NB makeMask now returns a value -1:1
+    alphaMask = makeMask(matrix.shape[0],shape,radius, center=(0.0,0.0), range=[0,1])
     return matrix*alphaMask
 
-def makeMask(matrixSize, shape='circle', radius=1.0, center=(0.0,0.0)):
+def makeMask(matrixSize, shape='circle', radius=1.0, center=(0.0,0.0), range=[-1,1]):
     """
     Returns a matrix to be used as an alpha mask (circle,gauss,ramp)
     
@@ -101,7 +104,9 @@ def makeMask(matrixSize, shape='circle', radius=1.0, center=(0.0,0.0)):
             outArray=makeGauss(rad,mean=0.0,sd=0.33333)
     else:
             raise ValueError('Unknown value for shape argument %s' % shape)
-    return outArray*2-1
+    mag=range[1]-range[0]
+    offset = range[0]
+    return outArray*mag + offset
 
 def makeRadialMatrix(matrixSize, center=(0.0,0.0), radius=1.0):
     """Generate a square matrix where each element val is
@@ -155,10 +160,10 @@ def conv2d(smaller, larger):
     Actually right now the matrices must be the same size (will sort out
     padding issues another day!)
     """
-    smallerFFT = numpy.fft2(smaller)
-    largerFFT = numpy.fft2(larger)
+    smallerFFT = fft2(smaller)
+    largerFFT = fft2(larger)
     
-    invFFT = numpy.ifft(smallerFFT*largerFFT)
+    invFFT = ifft2(smallerFFT*largerFFT)
     return invFFT.real
 
 def imfft(X):
