@@ -1,7 +1,7 @@
 """To build simple dialogues etc. (requires wxPython)
 """
 # Part of the PsychoPy library
-# Copyright (C) 2010 Jonathan Peirce
+# Copyright (C) 2011 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 from psychopy import log
 #from wxPython import wx
@@ -27,7 +27,8 @@ class Dlg(wx.Dialog):
         myDlg.addField('Age:', 21)
         myDlg.addText('Experiment Info')
         myDlg.addField('Grating Ori:',45)
-        if myDlg.show()==gui.OK:
+        myDlg.show()#show dialog and wait for OK or Cancel
+        if gui.OK:#then the user pressed OK
             thisInfo = myDlg.data
             print thisInfo
         else: print 'user cancelled'
@@ -51,46 +52,65 @@ class Dlg(wx.Dialog):
         #self.addText('')#insert some space at top of dialogue
         if pos==None:
             self.Center()
-    def addText(self, text):
+    def addText(self, text, color=''):
         textLength = wx.Size(8*len(text)+16, 25)
         myTxt = wx.StaticText(self,-1,
                                 label=text,
                                 style=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL,
                                 size=textLength)
+        if len(color): myTxt.SetForegroundColour(color)
         self.sizer.Add(myTxt,1,wx.ALIGN_CENTER)
         
-    def addField(self, label='', initial=''):
+    def addField(self, label='', initial='', color='', tip=''):
         """
-        Adds a (labelled) input field to the dialogue box
-        Returns a handle to the field (but not to the label).
+        Adds a (labelled) input field to the dialogue box, optional text color
+        and tooltip. Returns a handle to the field (but not to the label).
         """
         self.inputFieldNames.append(label)
         self.inputFieldTypes.append(type(initial))
         if type(initial)==numpy.ndarray:
             initial=initial.tolist() #convert numpy arrays to lists
+        container=wx.GridSizer(cols=2, hgap=10)
+        #create label
         labelLength = wx.Size(9*len(label)+16,25)#was 8*until v0.91.4
-        container=wx.BoxSizer(wx.HORIZONTAL)
         inputLabel = wx.StaticText(self,-1,label,
                                         size=labelLength,
-                                        style=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL)
+                                        style=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
+        if len(color): inputLabel.SetForegroundColour(color)
         container.Add(inputLabel, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
-        inputBox = wx.TextCtrl(self,-1,unicode(initial),size=(5*len(unicode(initial))+16,25))
-        container.Add(inputBox,1)
+        #create input control
+        if type(initial)==bool:
+            inputBox = wx.CheckBox(self, -1)
+            inputBox.SetValue(initial)
+        else:
+            inputLength = wx.Size(max(50, 5*len(unicode(initial))+16), 25)
+            inputBox = wx.TextCtrl(self,-1,unicode(initial),size=inputLength)
+        if len(color): inputBox.SetForegroundColour(color)
+        if len(tip): inputBox.SetToolTip(wx.ToolTip(tip))
+        
+        container.Add(inputBox,1, wx.ALIGN_CENTER_VERTICAL)
         self.sizer.Add(container, 1, wx.ALIGN_CENTER)
         
         self.inputFields.append(inputBox)#store this to get data back on OK
         return inputBox
     
-    def addFixedField(self,label='',value=''):
-        """Adds a field to the dialogue box (like addField) but
-        the field cannot be edited. e.g. Display experiment
-        version.
+    def addFixedField(self,label='',value='',tip=''):
+        """Adds a field to the dialogue box (like addField) but the field cannot
+        be edited. e.g. Display experiment version. tool-tips are disabled (by wx).
         """
-        thisField = self.addField(label,value)
-        thisField.Disable()
+        thisField = self.addField(label,value,color='Gray',tip=tip)
+        thisField.Disable() # wx disables tooltips too; we pass them in anyway
         return thisField
         
     def show(self):
+        """Presents the dialog and waits for the user to press either OK or CANCEL.
+        
+        This function returns nothing.
+        
+        When they do, dlg.OK will be set to True or False (according to which
+        button they pressed. If OK==True then dlg.data will be populated with a 
+        list of values coming from each of the input fields created. 
+        """
         #add buttons for OK and Cancel
         buttons = wx.BoxSizer(wx.HORIZONTAL)
         OK = wx.Button(self, wx.ID_OK, " OK ")
@@ -115,13 +135,8 @@ class Dlg(wx.Dialog):
                     exec("self.data.append("+thisVal+")")#evaluate it
                 elif thisType==numpy.ndarray:
                     exec("self.data.append(numpy.array("+thisVal+"))")
-                elif thisType in [str,unicode]:#a num array or string?
+                elif thisType in [str,unicode,bool]:
                     self.data.append(thisVal)
-                elif thisType==bool:#a num array or string?
-                    if thisVal in ['True','true']: self.data.append(True)
-                    elif thisVal in ['False','false']: self.data.append(False)
-                    else: 
-                        raise TypeError("%s should be 'True' or 'False' but '%s' was entered" %(thisName, thisVal))
                 else:
                     log.warning('unknown type:'+self.inputFieldNames[n])
                     self.data.append(thisVal)
@@ -154,21 +169,27 @@ class DlgFromDict(Dlg):
     If the user cancels (rather than pressing OK),
     then the dictionary remains unchanged. If you want to check whether
     the user hit OK, then check whether DlgFromDict.OK equals
-    True or False    
+    True or False
+    
+    See GUI.py for a usage demo, including order and tip (tooltip).
     """
-    def __init__(self, dictionary, title='',fixed=[]):
+    def __init__(self, dictionary, title='',fixed=[], order=[], tip={}):
         Dlg.__init__(self, title)
         self.dictionary=dictionary
         keys = self.dictionary.keys()
         keys.sort()
+        if len(order):
+            keys = order + list(set(keys).difference(set(order)))
         types=dict([])
         for field in keys:
             #DEBUG: print field, type(dictionary[field])
             types[field] = type(self.dictionary[field])
+            tooltip = ''
+            if field in tip.keys(): tooltip = tip[field]
             if field in fixed:
-                self.addFixedField(field,self.dictionary[field])
+                self.addFixedField(field,self.dictionary[field], tip=tooltip)
             else:
-                self.addField(field,self.dictionary[field])
+                self.addField(field,self.dictionary[field], tip=tooltip)
         #show it and collect data
         #tmp= wx.PySimpleApp()#this should have been done by Dlg ?
         self.show()
@@ -177,24 +198,35 @@ class DlgFromDict(Dlg):
                 self.dictionary[thisKey]=self.data[n]
         
 def fileSaveDlg(initFilePath="", initFileName="", 
-                prompt="Select file to save"):
+                prompt="Select file to save",
+                allowed=None):
     """A simple dialogue allowing access to the file system.
     (Useful in case you collect an hour of data and then try to 
     save to a non-existent directory!!)
     
-    usage:
-        validPathName = fileSaveDlg(initFilePath,initFileName)
-    
+    :parameters:
+        initFilePath: string
+            default file path on which to open the dialog
+        initFilePath: string
+            default file name, as suggested file
+        prompt: string (default "Select file to open")
+            can be set to custom prompts
+        allowed: string
+            a string to specify file filters. 
+            e.g. "BMP files (*.bmp)|*.bmp|GIF files (*.gif)|*.gif"
+            See http://www.wxpython.org/docs/api/wx.FileDialog-class.html for further details
+            
     If initFilePath or initFileName are empty or invalid then
     current path and empty names are used to start search.
     
     If user cancels the None is returned.
     """
-    allowed = "All files (*.*)|*.*"  #\
+    if allowed==None:
+        allowed = "All files (*.*)|*.*"  #\
             #"txt (*.txt)|*.txt" \
             #"pickled files (*.pickle, *.pkl)|*.pickle" \
             #"shelved files (*.shelf)|*.shelf"
-    tmpApp = wx.PySimpleApp()	
+    tmpApp = wx.PySimpleApp()
     dlg = wx.FileDialog(None,prompt, 
                           initFilePath, initFileName, allowed, wx.SAVE)
     if dlg.ShowModal() == OK:
@@ -209,25 +241,36 @@ def fileSaveDlg(initFilePath="", initFileName="",
 
 def fileOpenDlg(tryFilePath="",
                 tryFileName="",
-                prompt="Select file to open"):
+                prompt="Select file to open",
+                allowed=None):
     """A simple dialogue allowing access to the file system.
     (Useful in case you collect an hour of data and then try to 
     save to a non-existent directory!!)
     
-    usage:
-        validPathName = fileSaveDlg(initFilePath,initFileName)
-    
-    If initFilePath or initFileName are empty or invalid then
+    :parameters:
+        tryFilePath: string
+            default file path on which to open the dialog
+        tryFilePath: string
+            default file name, as suggested file
+        prompt: string (default "Select file to open")
+            can be set to custom prompts
+        allowed: string (available since v1.62.01)
+            a string to specify file filters. 
+            e.g. "BMP files (*.bmp)|*.bmp|GIF files (*.gif)|*.gif"
+            See http://www.wxpython.org/docs/api/wx.FileDialog-class.html for further details
+            
+    If tryFilePath or tryFileName are empty or invalid then
     current path and empty names are used to start search.
     
-    If user cancels the None is returned.
+    If user cancels, then None is returned.
     """
-    allowed = "PsychoPy Data (*.psydat)|*.psydat|"\
-            "txt (*.txt,*.dlm)|*.txt|" \
+    if allowed==None:
+        allowed = "PsychoPy Data (*.psydat)|*.psydat|"\
+            "txt (*.txt,*.dlm,*.csv)|*.txt;*.dlm;*.csv|" \
             "pickled files (*.pickle, *.pkl)|*.pickle|" \
             "shelved files (*.shelf)|*.shelf|" \
             "All files (*.*)|*.*"
-    tmpApp = wx.PySimpleApp()	
+    tmpApp = wx.PySimpleApp()
     dlg = wx.FileDialog(None, prompt,
                           tryFilePath, tryFileName, allowed, wx.OPEN|wx.FILE_MUST_EXIST|wx.MULTIPLE)
     if dlg.ShowModal() == OK:
