@@ -1,5 +1,5 @@
 # Part of the PsychoPy library
-# Copyright (C) 2010 Jonathan Peirce
+# Copyright (C) 2011 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 """
@@ -339,7 +339,10 @@ class QuicktimeMovie(object):
         if self.movie is None:
             self._movTmpFileName=os.path.join(tempfile.gettempdir(), 'psychopyTmp.mov')#could this line fail if no permissions?
             #self._movTmpFileName='tmpMov'#this is handy if we want to inspect the temp file
-            self.movie, err=QTKit.QTMovie.alloc().initToWritableFile_error_(self._movTmpFileName)#
+            try:
+                self.movie, err=QTKit.QTMovie.alloc().initToWritableFile_error_(self._movTmpFileName,None)#
+            except:
+                self.movie, err=QTKit.QTMovie.alloc().initToWritableFile_error_(self._movTmpFileName)#under some versions (10.5?) this call is different
             if err is not None:
                 print str(err)
             self.movie.setEditable_(True)
@@ -373,12 +376,12 @@ class QuicktimeMovie(object):
         os.remove(tmpFileName)
         self.frameN += 1
         
-    def save(self, filename=None):
+    def save(self, filename=None, compressed=True):
         """Save the movie to the self.filename or to a new one if given
         """
         if filename==None: filename = self.filename
         self.movie.writeToFile_withAttributes_(filename,
-            {QTKit.QTMovieFlatten:True})#if True then there is no inter-frame compression
+            {QTKit.QTMovieFlatten:(not compressed)})#if True then there is no inter-frame compression
         #self.movie.updateMovieFile()#this doesn't use QTExport settings and you end up with large files (uncompressed in time)
     def __del__(self):
         """Remove any tmp files if possible (including any from previous runs that garbage collection wasn't able to collect)
