@@ -24,7 +24,7 @@ pyaudio:
     
 """
 # Part of the PsychoPy library
-# Copyright (C) 2010 Jonathan Peirce
+# Copyright (C) 2011 Jonathan Peirce
 # Distributed under the terms of the GNU General Public License (GPL).
 
 import numpy, threading, time
@@ -32,6 +32,7 @@ from os import path
 from string import capitalize
 from sys import platform, exit, stdout
 from psychopy import event, core, log
+from psychopy.constants import *
 
 if platform=='win32':
     mediaLocation="C:\\Windows\Media"
@@ -46,6 +47,7 @@ audioAPI=None
 try:
     import pyglet
     pyglet.options['audio'] = ('silent')
+    pyglet.options['debug_gl'] = False
     import pyglet.media.procedural
     import pyglet.media#, pyglet.resource
     import ctypes, math
@@ -72,40 +74,54 @@ class _SoundBase:
     """
     def __init__(self,value="C",secs=0.5,octave=4, sampleRate=44100, bits=16):
         """
-        value: can be a number, string or an array.
         
-            If it's a number between 37 and 32767 then a tone will be generated at
-            that frequency in Hz.
-            -----------------------------
-            It could be a string for a note ('A','Bfl','B','C','Csh'...) 
-            - you may want to specify which octave as well
-            -----------------------------
-            Or a string could represent a filename in the current
-            location, or mediaLocation, or a full path combo. 
-            At the moment only .wav and .ogg files are supported (but others might be added
-            if we can create numpy arrays out of pyglet's sound files)
-            -----------------------------
-            Or by giving an Nx2 numpy array of floats (-1:1) you 
-            can specify the sound yourself as a waveform
+        :parameters:
+            value: can be a number, string or an array:        
+                * If it's a number between 37 and 32767 then a tone will be generated at that frequency in Hz.
+                * It could be a string for a note ('A','Bfl','B','C','Csh'...). Then you may want to specify which octave as well
+                * Or a string could represent a filename in the current location, or mediaLocation, or a full path combo
+                * Or by giving an Nx2 numpy array of floats (-1:1) you can specify the sound yourself as a waveform
+                
+            secs: is only relevant if the value is a note name or
+                a frequency value
+                
+            octave: is only relevant if the value is a note name.
+                Middle octave of a piano is 4. Most computers won't
+                output sounds in the bottom octave (1) and the top
+                octave (8) is generally painful
+                
+            sampleRate(=44100): only used for sounds using pyglet. Pygame uses one rate for all sounds
+                sample rate for all sounds (once initialised) 
             
-        secs: is only relevant if the value is a note name or
-            a frequency value
-            
-        octave: is only relevant if the value is a note name.
-            Middle octave of a piano is 4. Most computers won't
-            output sounds in the bottom octave (1) and the top
-            octave (8) is generally painful
-            
-        sampleRate(=44100): only used for sounds using pyglet. Pygame uses one rate for all sounds
-            sample rate for all sounds (once initialised) 
-        
-        bits(=16): Only 8- and 16-bits supported so far.
-            Only used for sounds using pyglet. Pygame uses the same
-            sample rate for all sounds (once initialised) 
-        """        
-        #try to determine what the sound is
+            bits(=16): Only 8- and 16-bits supported so far.
+                Only used for sounds using pyglet. Pygame uses the same
+                sample rate for all sounds (once initialised) 
+        """
         self._snd=None
-        if type(value) is str:
+        self.setSound(value=value, secs=secs, octave=octave)
+        
+    def setSound(self, value, secs=0.5, octave=4):
+        """Set the sound to be played. 
+        
+        Often this is not needed byt the user - it is called implicitly during
+        initialisation.
+        
+        :parameters:
+        
+            value: can be a number, string or an array:        
+                * If it's a number between 37 and 32767 then a tone will be generated at that frequency in Hz.
+                * It could be a string for a note ('A','Bfl','B','C','Csh'...). Then you may want to specify which octave as well
+                * Or a string could represent a filename in the current location, or mediaLocation, or a full path combo
+                * Or by giving an Nx2 numpy array of floats (-1:1) you can specify the sound yourself as a waveform
+                
+            secs: duration (only relevant if the value is a note name or a frequency value)
+                
+            octave: is only relevant if the value is a note name.
+                Middle octave of a piano is 4. Most computers won't
+                output sounds in the bottom octave (1) and the top
+                octave (8) is generally painful
+        """
+        if type(value) in [str, unicode]:
             #try to open the file
             OK = self._fromNoteName(value,secs,octave)
             #or use as a note name
@@ -120,7 +136,8 @@ class _SoundBase:
             self._fromArray(value)
         if self._snd is None:
             raise RuntimeError, "I dont know how to make a "+value+" sound"
-            
+        self.status=NOT_STARTED
+        
     def play(self, fromStart=True):
         """Starts playing the sound on an available channel. 
         If no sound channels are available, it will not play and return None. 
@@ -159,15 +176,19 @@ class _SoundBase:
             'Dfl' : -8,
             'D' : -7,
             'Dsh' : -6,
+            'Efl' : -6,
             'E' : -5,
             'F' : -4,
             'Fsh' : -3,
+            'Gfl' : -3,
             'G' : -2,
             'Gsh' : -1,
+            'Afl': -1,
             'A': 0,
             'Ash':+1,
             'Bfl': +1,
             'B': +2,
+            'Bsh': +2,
             }
         if thisNote not in stepsFromA.keys():
             return False
@@ -216,7 +237,7 @@ class SoundPygame(_SoundBase):
 
         #check initialisation
         if not mixer.get_init():
-            pygame.mixer.init(22050, -16, 2, 3072)
+            pygame.mixer.init(sampleRate, -16, 2, 3072)
         
         inits = mixer.get_init()
         if inits is None:
@@ -224,24 +245,10 @@ class SoundPygame(_SoundBase):
             inits = mixer.get_init()                
         self.sampleRate, self.format, self.isStereo = inits
         
-        #try to determine what the sound is
+        #try to create sound
         self._snd=None
-        if type(value) in [str, unicode]:
-            #try to open the file
-            OK = self._fromNoteName(value,secs,octave)
-            #or use as a note name
-            if not OK: self._fromFile(value)
-            
-        elif type(value) in [float,int]:
-            #we've been asked for a particular Hz
-            self._fromFreq(value, secs)
-            
-        elif type(value) in [list,numpy.ndarray]:
-            #create a sound from the input array/list
-            self._fromArray(value)
-        if self._snd is None:
-            raise RuntimeError, "I dont know how to make a "+value+" sound"
-            
+        self.setSound(value=value, secs=secs, octave=octave)
+        
     def play(self, fromStart=True):
         """Starts playing the sound on an available channel. 
         If no sound channels are available, it will not play and return None. 
@@ -253,18 +260,18 @@ class SoundPygame(_SoundBase):
         be played over each other.
         """
         self._snd.play()
-        
+        self.status=STARTED
     def stop(self):
         """Stops the sound immediately"""
         self._snd.stop()
-                    
+        self.status=STOPPED
     def fadeOut(self,mSecs):
         """fades out the sound (when playing) over mSecs.
         Don't know why you would do this in psychophysics but it's easy
         and fun to include as a possibility :)
         """
         self._snd.fadeout(mSecs)
-        
+        self.status=STOPPED
     def getDuration(self):
         """Get's the duration of the current sound in secs"""
         return self._snd.get_length()
@@ -298,7 +305,7 @@ class SoundPygame(_SoundBase):
         #get a mixer.Sound object from an array of floats (-1:1)
         
         #make stereo if mono
-        if self.isStereo and \
+        if self.isStereo==2 and \
             (len(thisArray.shape)==1 or thisArray.shape[1]<2):
             tmp = numpy.ones((len(thisArray),2))
             tmp[:,0] = thisArray
@@ -362,24 +369,10 @@ class SoundPyglet(_SoundBase):
         
         #self._player._eos_action='pause'
         self._player._on_eos=self._onEOS
-            
-        #try to determine what the sound is
+        
+        #try to create sound
         self._snd=None
-        if type(value) in [str, unicode]:
-            #try to open the file
-            OK = self._fromNoteName(value,secs,octave)
-            #or use as a note name
-            if not OK: self._fromFile(value)
-            
-        elif type(value) in [float,int]:
-            #we've been asked for a particular Hz
-            self._fromFreq(value, secs)
-            
-        elif type(value) in [list,numpy.ndarray]:
-            #create a sound from the input array/list
-            self._fromArray(value)
-        if self._snd is None:
-            raise RuntimeError, "I dont know how to make a "+value+" sound"
+        self.setSound(value=value, secs=secs, octave=octave)
             
     def play(self, fromStart=True):
         """Starts playing the sound on an available channel. 
@@ -393,6 +386,7 @@ class SoundPyglet(_SoundBase):
         """
         self._player.play()
         pyglet.media.dispatch_events()
+        self.status=STARTED
 
     def _onEOS(self):
         self._player._playing = False
@@ -401,11 +395,13 @@ class SoundPyglet(_SoundBase):
         self._player.queue(self._snd)
         self._player._fill_audio()
         self._player.dispatch_event('on_eos')
+        self.status=FINISHED
         return True
         
     def stop(self):
         """Stops the sound immediately"""
         self._snd._stop()
+        self.status=STOPPED
             
     def getDuration(self):
         s=self._snd      
@@ -509,24 +505,9 @@ class SoundPyaudio(_SoundBase):
         self._thread = PyAudioThread(self, pollingPeriod=0.01)
         self._thread.start()
         
-        #try to determine what the sound is
+        #try to create sound
         self._snd=None
-        if type(value) in [str, unicode]:
-            #try to open the file
-            OK = self._fromNoteName(value,secs,octave)
-            #or use as a note name
-            if not OK: #hopefully a file
-                self._fromFile(value)
-                
-        elif type(value) in [float,int]:
-            #we've been asked for a particular Hz
-            self._fromFreq(value, secs)
-            
-        elif type(value) in [list,numpy.ndarray]:
-            #create a sound from the input array/list
-            self._fromArray(value)
-        if self.rawData is None:
-            raise RuntimeError, "I dont know how to make a "+value+" sound"
+        self.setSound(value=value, secs=secs, octave=octave)
             
         if self.bits==16: paFormat = pyaudio.paInt16
         elif self.bits==8: paFormat = pyaudio.paInt8
@@ -812,7 +793,7 @@ def initPygame(rate=22050, bits=16, stereo=True, buffer=1024):
     The format cannot be changed once initialised or once a Window has been created. 
     
     If a Sound object is created before this function is run it will be
-    executed with default format (signed 16bit stereo at 44KHz).
+    executed with default format (signed 16bit stereo at 22KHz).
     
     For more details see pygame help page for the mixer.
     """
